/*NSRCOPYRIGHT
	Copyright (C) 1999-2011 University of Washington
	Developed by the National Simulation Resource
	Department of Bioengineering,  Box 355061
	University of Washington, Seattle, WA 98195-5061.
	Dr. J. B. Bassingthwaighte, Director
END_NSRCOPYRIGHT*/

// Data Generator script

package JSim.datagen;

import JSim.util.*;
import JSim.expr.*;
import JSim.data.*;
import java.util.*;

public class DGCode {
	private ArrayList<DGTask> tasks;
	private LinkedHashMap<String, DGTask> tasksByName;
	private ArrayList<DGDataGen> datagens;
	private LinkedHashMap<String, DGDataGen> datagensByName;

	// constructor
	public DGCode() {
	    tasks = new ArrayList<DGTask>();
	    tasksByName = new LinkedHashMap<String, DGTask>();
	    datagens = new ArrayList<DGDataGen>();
	    datagensByName = new LinkedHashMap<String, DGDataGen>();
	}

	// add stuff
	protected void add(DGTask task) throws Xcept {
	    String name = task.name();
	    if (task(name) != null) throw new Xcept(
	    	"Duplicate tasks with name <" + name + ">");
	    tasks.add(task);
	    tasksByName.put(name, task);
	}
	protected void add(DGDataGen datagen) throws Xcept {
	    String name = datagen.name();
	    if (datagen(name) != null) throw new Xcept(
	    	"Duplicate datagens with name <" + name + ">");
	    datagens.add(datagen);
	    datagensByName.put(name, datagen);
	}
	
	// query
	public int ntasks() { return tasks.size(); }
	public DGTask task(String n) { return tasksByName.get(n); }
	public DGTask task(int i) { return tasks.get(i); };
	public int ndatagens() { return datagens.size(); }
	public DGDataGen datagen(String n) { return datagensByName.get(n); }
	public DGDataGen datagen(int i) { return datagens.get(i); };

	// get formatted text
	public String getText() {
	    StringBuffer buf = new StringBuffer();
	    buf.append("DataGen version 1.1\n");
	    buf.append("\n");
	    for (int i=0; i<ntasks(); i++) {
	    	DGTask task = task(i);
	    	buf.append("task " + task.name() + "(" +
		    task.modelName);
		if (task.parsetName == null) 
		    buf.append(", " + task.parsetName);
		buf.append(")");
		if (task.nassigns() > 0) {
		    buf.append(" {\n");
		    for (int j=0; j<task.nassigns(); j++) 
		    	buf.append("\t" + task.assign(j) + "\n");
		    buf.append("}\n");
		} else {
		    buf.append(";");
		}
	    }
	    buf.append("\n");
	    for (int i=0; i<ndatagens(); i++)
	    	buf.append(datagen(i).toString() + "\n");
	    return buf.toString();
	}

}
