/*NSRCOPYRIGHT
	Copyright (C) 1999-2008 University of Washington
	Developed by the National Simulation Resource
	Department of Bioengineering,  Box 355061
	University of Washington, Seattle, WA 98195-5061.
	Dr. J. B. Bassingthwaighte, Director
END_NSRCOPYRIGHT*/

// DataGen scanner

package JSim.datagen;

import JSim.util.*;
import JSim.expr.*;
import java_cup.runtime.Symbol;

%%
%class DGCodeScanner
%cup
%line
%char
%{
	// code to include in Yylex class
	public int getLineNo() { return yyline+1; }
	public int getCharNo() { return yychar; }
	public String tokText() { return new String(yytext()); }
	public void lexError() throws Xcept {
	    throw new Xcept("Illegal character");
	}
%}

%eofval{
	return new Symbol(DGCodeParserSym.EOF);
%eofval}
 
%yylexthrow{
	Xcept
%yylexthrow}

%%
"{{" ~"}}" { /* double brace enclosed code blocks */
	   String s = tokText();
	   s = s.substring(2, s.length()-2);
	   return new Symbol(DGCodeParserSym.STRING, s);
	}
"//".*\n { /* ignore Unix comments */ }
"//".*\r { /* ignore DOS comments */ }
"/*" ~"*/" { /* ignore C-style comments */ }

"DGL"   { return new Symbol(DGCodeParserSym.DATAGENCODE); }
"version" { return new Symbol(DGCodeParserSym.VERSION); }
"unit" { return new Symbol(DGCodeParserSym.UNIT); }
"import" { return new Symbol(DGCodeParserSym.IMPORT); }
"converson" { return new Symbol(DGCodeParserSym.CONVERSION); }
"input" { return new Symbol(DGCodeParserSym.INPUT); }
"output" { return new Symbol(DGCodeParserSym.OUTPUT); }
"task" { return new Symbol(DGCodeParserSym.TASK); }
"assign" { return new Symbol(DGCodeParserSym.ASSIGN); }

\"[^\"]*\" { String s = tokText();
	   s = s.substring(1, s.length()-1);
	   return new Symbol(DGCodeParserSym.STRING, s); 
	}

"PI"     { return new Symbol(DGCodeParserSym.PI, tokText()); }
"true"   { return new Symbol(ExprParserSym.TRUE, tokText()); }
"false"  { return new Symbol(ExprParserSym.FALSE, tokText()); }
"="      { return new Symbol(DGCodeParserSym.EQ, tokText()); }
"<>"     { return new Symbol(DGCodeParserSym.NE, tokText()); }
"<"      { return new Symbol(DGCodeParserSym.LT, tokText()); }
">"      { return new Symbol(DGCodeParserSym.GT, tokText()); }
"<="     { return new Symbol(DGCodeParserSym.LE, tokText()); }
">="     { return new Symbol(DGCodeParserSym.GE, tokText()); }
"if"     { return new Symbol(DGCodeParserSym.IF, tokText()); }
"else"   { return new Symbol(DGCodeParserSym.ELSE, tokText()); }
"and"    { return new Symbol(DGCodeParserSym.AND, tokText()); }
"or"     { return new Symbol(DGCodeParserSym.OR, tokText()); }

([A-Za-z])[0-9A-Za-z_.:]* { return new Symbol(DGCodeParserSym.IDENT, tokText()); }
([0-9]+)?"."?[0-9]+([Ee]([+-])?[0-9]+)? { return new Symbol(DGCodeParserSym.NUMBER, tokText()); }
[0-9]+"."?([0-9]+)?([Ee]([+-])?[0-9]+)? { return new Symbol(DGCodeParserSym.NUMBER, tokText()); }

"+" { return new Symbol(DGCodeParserSym.PLUS); }
"-" { return new Symbol(DGCodeParserSym.MINUS); }
"^" { return new Symbol(DGCodeParserSym.POWER); }
"*" { return new Symbol(DGCodeParserSym.TIMES); }
"/" { return new Symbol(DGCodeParserSym.DIVIDE); }

"(" { return new Symbol(DGCodeParserSym.LPAREN); }
")" { return new Symbol(DGCodeParserSym.RPAREN); }
"{" { return new Symbol(DGCodeParserSym.LBRACE); }
"}" { return new Symbol(DGCodeParserSym.RBRACE); }
";" { return new Symbol(DGCodeParserSym.SEMI); }
"," { return new Symbol(DGCodeParserSym.COMMA); }

[ \t\r\f\n] { /* ignore white space. */ }
. { lexError(); }
