/*NSRCOPYRIGHT
	Copyright (C) 1999-2011 University of Washington
	Developed by the National Simulation Resource
	Department of Bioengineering,  Box 355061
	University of Washington, Seattle, WA 98195-5061.
	Dr. J. B. Bassingthwaighte, Director
END_NSRCOPYRIGHT*/

// translate between various model text formats

package JSim.tests;

import org.w3c.dom.*;
import java.util.*;
import java.io.*;
import java.net.URL;

import JSim.util.*;
import JSim.expr.*;
import JSim.data.*;
import JSim.mml.*;
import JSim.mathml.*;
import JSim.plan.*;
import JSim.sbml.*;
import JSim.cellml.*;
import JSim.aserver.*;

public class TranslatorTest {
		
	// test harness
	public static final void main(String[] args) throws Xcept {

	    // parse switches
	    String jsimHome = null;
	    String options = null;
	    String buildDir = null;
	    int i = 0;
	    while (i<args.length && args[i].startsWith("-")) {
	    	if (args[i].equals("-home"))
		    jsimHome = args[++i];
		else if (args[i].equals("-bd"))
		    buildDir = args[++i];
		else if (args[i].equals("-o"))
		    options = args[++i];
		else throw new Xcept(
		    "Unknown switch: " + args[i]);
		i++;  
	    }

	    // parse args
	    if (args.length - i != 3) throw new Xcept(
	    	"Usage: TranslatorTest [-home JSIMHOME] [-bd buildDir] [-o options] srcType destType file\n" 
		+ " MML=" + ASModel.TEXT_MML + "\n"
		+ " XML=" + ASModel.TEXT_XML + "\n"
		+ " SBML=" + ASModel.TEXT_SBML + "\n"
		+ " CellML=" + ASModel.TEXT_CELLML + "\n"
		+ " XMML=" + ASModel.TEXT_XMML);	 
	    int srcType = Util.toInt(args[i++]);
	    int destType = Util.toInt(args[i++]);
	    File f = new File(args[i++]);
	    String srcText = UtilIO.readText(f);
	    
	    // create server
	    if (jsimHome != null)
	    	System.setProperty("jsim.home", jsimHome);
	    NamedVal.NList soptions = new NamedVal.NList();
	    if (buildDir != null)
	    	soptions.setVal("buildDir", buildDir);
	    ASServer server = ASServer.create(soptions, null, null);	    
	
	    // translate text
	    try {
	    	String out = server.translateModelText(srcType, destType, srcText, options);
	    	System.out.println(out);
	    } catch (Exception e) {
	    	e.printStackTrace();
	    }
	    server.disconnect();
	}
}
	

